from datetime import timedelta

from django.utils import timezone
from rest_framework.exceptions import ValidationError
from rest_framework.generics import get_object_or_404

from drf_auth_service.models import ActivationCode
from drf_auth_service.settings import settings


def validate_token_by_type(token, code_type, **additional_params):
    if not settings.DEBUG_MODE or code_type == ActivationCode.CodeTypes.RESET_PASSWORD:
        password_reset_token_validation_time = 24

        try:
            reset_password_token = get_object_or_404(ActivationCode, key=token, key_type=code_type, **additional_params)
        except ActivationCode.DoesNotExist:
            raise ValidationError(['Invalid Token'])

        expiry_date = reset_password_token.created_at + timedelta(hours=password_reset_token_validation_time)

        if timezone.now() > expiry_date:
            reset_password_token.delete()
            raise ValidationError(['Token expired'])
    else:
        if token != settings.DEBUG_KEY: # noqa
            raise ValidationError(['Invalid Token'])
    return token



from drf_auth_service.common.permissions import ServiceTokenPermission

AUTHENTICATION_PERMISSIONS = dict(
    register=[ServiceTokenPermission],
    send_reset_password=[ServiceTokenPermission],
    reset_password_confirm=[ServiceTokenPermission],
    default=[ServiceTokenPermission]
)



from drf_auth_service.common.permissions import ServiceTokenPermission, SecretServiceTokenPermission

USER_PERMISSIONS = dict(
    user_confirm=[ServiceTokenPermission],
    resend_confirmation=[SecretServiceTokenPermission],
    destroy=[SecretServiceTokenPermission],
    block_user=[SecretServiceTokenPermission],
    list=[SecretServiceTokenPermission],
    unblock_user=[SecretServiceTokenPermission],
    default=[SecretServiceTokenPermission]
)



from django.core.exceptions import ImproperlyConfigured
from django.template.loader import render_to_string

from drf_auth_service.common.email_backends import HtmlTemplateBackend
from drf_auth_service.common.register_backends import PhoneCodeBackend
from drf_auth_service.common.sms_backends import TwilioBackend
from drf_auth_service.models import ActivationCode


class CustomHtmlTemplateBackend(HtmlTemplateBackend):
    def send(self, html_template, **context):
        try:
            email_html_message = render_to_string((html_template, context.get('default_template')), context)
        except ImproperlyConfigured:
            pass
        print('was successfully success')

    def send_reset_password(self, user):
        ActivationCode.create_token(user, ActivationCode.CodeTypes.RESET_PASSWORD)


class CustomTwilioBackend(TwilioBackend):
    def send(self, to_phone, message, from_phone=None):
        print(to_phone, message, 'success')


class CustomPhoneCodeBackend(PhoneCodeBackend):
    def register(self):
        user, code = self.register_user()

        return dict(code=code)



from unittest import mock

from djet import assertions
from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import APITestCase

from drf_auth_service.models import Service, SSOUser
from drf_auth_service_tests.test_app.backends import CustomHtmlTemplateBackend, CustomTwilioBackend


class ResetPasswordTest(
    APITestCase,
    assertions.StatusCodeAssertionsMixin,
    assertions.EmailAssertionsMixin,
    assertions.InstanceAssertionsMixin
):
    fixtures = [
        'test_app/fixtures/init_data.json',
    ]

    def setUp(self) -> None:
        self.password_raw = 'testovici'
        self.service_header = {"HTTP_SERVICE_TOKEN": Service.objects.first().public_token}

    @mock.patch("drf_auth_service.common.email_backends.HtmlTemplateBackend", CustomHtmlTemplateBackend)
    def test_send_reset_password_email(self):
        self.check_reset_password("tester@gmail.com")
        self.confirm_reset_password("tester@gmail.com")

    @mock.patch("drf_auth_service.common.sms_backends.TwilioBackend", CustomTwilioBackend)
    def test_send_reset_password_phone_number(self):
        self.check_reset_password("37361111111")
        self.confirm_reset_password("37361111111")

    @mock.patch("drf_auth_service.common.sms_backends.TwilioBackend", CustomTwilioBackend)
    def test_send_reset_password_phone_number_code(self):
        self.check_reset_password("37361111112")
        self.confirm_reset_password("37361111112")

    def check_reset_password(self, username):
        data = {"username": username}
        response = self.client.post(reverse("authentication-reset-password"), data, **self.service_header)
        self.assert_status_equal(response, status.HTTP_200_OK)

    def confirm_reset_password(self, username):
        token = SSOUser.objects.get(username=username).activation_code.first().key
        data = {
            "username": username,
            "password": self.password_raw,
            "confirm_password": self.password_raw,
            "token": token
        }
        response = self.client.post(reverse("authentication-reset-password-confirm"), data, **self.service_header)
        self.assert_status_equal(response, status.HTTP_200_OK)
        response = self.client.post(reverse("authentication-login"), data, **self.service_header)
        self.assert_status_equal(response, status.HTTP_200_OK)




from djet import assertions
from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import APITestCase

from drf_auth_service.models import Service


class ServiceTest(
    APITestCase,
    assertions.StatusCodeAssertionsMixin,
    assertions.EmailAssertionsMixin,
    assertions.InstanceAssertionsMixin
):
    fixtures = [
        'test_app/fixtures/init_data.json',
    ]

    def test_service_crud(self):
        response = self.client.post(reverse("service-list"), {"name": "test"})
        service_id = response.json()['id']
        self.assert_status_equal(response, status.HTTP_201_CREATED)
        self.assert_instance_exists(Service, name="test")
        response = self.client.get(reverse("service-detail", args=(service_id,)))
        self.assertEqual(response.json()['name'], 'test')
        self.assert_status_equal(response, status.HTTP_200_OK)
        response = self.client.patch(reverse("service-detail", args=(1,)), {"name": "another_test"})
        self.assert_status_equal(response, status.HTTP_200_OK)
        self.assertTrue(Service.objects.filter(name='another_test').exists())
        response = self.client.get(reverse("service-configs", args=(1,)))
        self.assert_status_equal(response, status.HTTP_200_OK)
        response = self.client.patch(reverse("service-configs", args=(1,)), {"send_confirmation": True})
        self.assertTrue(response.json()["send_confirmation"])
        self.assert_status_equal(response, status.HTTP_200_OK)
        response = self.client.delete(reverse("service-detail", args=(1,)))
        self.assert_status_equal(response, status.HTTP_204_NO_CONTENT)
        self.assertFalse(Service.objects.filter(name='another_test').exists())




# backends name

MAILCHIMP_BACKEND_NAME = 'mailchimp'
SENDINBLUE_BACKEND_NAME = 'sendinblue'
HTML_BACKEND_NAME = 'html_email'
TWILIO_BACKEND_NAME = 'twilio'

# Backends types

SMS_BACKEND = "sms_backend"
EMAIL_BACKEND = "email_backend"
